"use strict";
/**
 * @file 开发环境webpack配置
 * @author zhanglili
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack_merge_1 = __importDefault(require("webpack-merge"));
const proxy_agent_1 = __importDefault(require("proxy-agent"));
const internal_ip_1 = __importDefault(require("internal-ip"));
const createAgent = (possibleProxyURL) => {
    if (possibleProxyURL) {
        return new proxy_agent_1.default(possibleProxyURL);
    }
    return undefined;
};
exports.default = ({ proxyDomain }, devServerSettings, addition = {}, openBrowser) => {
    const { apiPrefixes = [], defaultProxyDomain = '', https, port, hot = 'none', openPage = '', onStart, } = devServerSettings;
    const devHost = internal_ip_1.default.v4.sync();
    const urlPrefix = (https ? 'https://' : 'http://') + (proxyDomain || defaultProxyDomain);
    const agent = createAgent(process.env[https ? 'https_proxy' : 'http_proxy']);
    const proxy = apiPrefixes.reduce((proxy, prefix) => ({ ...proxy, [prefix]: { agent, target: urlPrefix, changeOrigin: true } }), {});
    const baseConfig = {
        port,
        proxy,
        openPage,
        disableHostCheck: true,
        host: '0.0.0.0',
        public: `${devHost}:${port}`,
        open: openBrowser,
        quiet: true,
        compress: true,
        inline: true,
        hot: hot === 'simple',
        hotOnly: hot === 'all',
        publicPath: '/assets/',
        stats: 'normal',
        historyApiFallback: {
            rewrites: [
                {
                    from: /^\/\w+\.html/,
                    to(context) {
                        return `/assets${context.parsedUrl.href}`;
                    },
                },
                {
                    from: /./,
                    to: '/assets/index.html',
                },
            ],
            disableDotRule: true,
        },
        before: onStart,
    };
    const mergedConfig = webpack_merge_1.default({ devServer: baseConfig }, { devServer: addition });
    return mergedConfig.devServer;
};
//# sourceMappingURL=config.js.map