"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const hasha_1 = __importDefault(require("hasha"));
const lodash_1 = require("lodash");
const webpack_1 = __importDefault(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const build_1 = require("../../../lib/build");
const project_1 = require("../../../lib/project");
const config_1 = __importDefault(require("./config"));
const startDevServer = async (cmd, fresh) => {
    const projectSettings = project_1.readProjectSettings(cmd, 'dev');
    const { name: hostPackageName } = project_1.readHostPackageConfig(cmd.cwd);
    const entry = {
        hostPackageName,
        usage: 'devServer',
        mode: cmd.mode || 'development',
        cwd: cmd.cwd,
        srcDirectory: cmd.src,
        buildTarget: cmd.buildTarget || 'dev',
        features: projectSettings.featureMatrix[cmd.buildTarget],
        isDefaultTarget: true,
        buildVersion: 'HEAD',
        entries: [],
        analyze: false,
        buildTime: (new Date()).toISOString(),
        ...projectSettings,
        devServer: { ...projectSettings.devServer },
    };
    const config = build_1.createDevConfig(entry);
    const devServerConfig = config_1.default(cmd, entry.devServer, config.devServer, fresh);
    webpack_dev_server_1.default.addDevServerEntrypoints(config, devServerConfig);
    const compiler = webpack_1.default(config);
    const server = new webpack_dev_server_1.default(compiler, devServerConfig);
    server.listen(devServerConfig.port);
    return server;
};
const computeSettingsHash = (cwd) => {
    const settings = fs.readFileSync(path.join(cwd, 'settings.js'));
    return hasha_1.default(settings);
};
exports.default = async (cmd) => {
    process.env.NODE_ENV = cmd.mode || 'development';
    const { cwd } = cmd;
    let startingServer = startDevServer(cmd, true);
    const settingsLocation = path.join(cwd, 'settings.js');
    if (!fs.existsSync(settingsLocation)) {
        return;
    }
    let settingsHash = computeSettingsHash(cwd);
    let nextStart = null;
    fs.watch(settingsLocation, async () => {
        // `fs.watch`是不稳定的，一次修改会触发多次，因此用hash做一下比对
        const newSettingsHash = computeSettingsHash(cwd);
        if (newSettingsHash === settingsHash) {
            return;
        }
        settingsHash = newSettingsHash;
        console.log('Detected settings.js change, restarting dev server...');
        if (nextStart) {
            return;
        }
        nextStart = () => {
            startingServer = startDevServer(cmd, false);
            nextStart = null;
        };
        const server = await startingServer;
        server.close(nextStart || lodash_1.noop);
    });
};
//# sourceMappingURL=run.js.map