"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const lockfile_1 = require("@yarnpkg/lockfile");
const project_1 = require("../../../lib/project");
// `yarn.lock`在解析以后，大致的结构如下：
//
// {
//     "@babel/core@^7.0.0": {
//         "version": "7.1.2",
//         "resolved": "https://...",
//         "dependencies": {
//             "@babel/parser": "^7.0.3",
//             "@babylon": "^7.2.0"
//         }
//     }
// }
//
// 需要注意的是，每一个键（以及`dependencies`里的键）中的版本是一个semver的范围，而不是精确的版本号。
//
// 根据`dependencies`里的名称和版本号组成新的键，可以在整个lock文件里继续找到那个包的信息，以此递归来解析出整个依赖树。
const collectDependenciesFrom = (lock) => {
    const collect = (name, versionRange, met = new Set()) => {
        const { version, dependencies } = lock[name + '@' + versionRange];
        const identifier = name + '@' + version;
        // 会循环引用，所以拿一个`Set`处理
        if (met.has(identifier)) {
            return { name, version, circular: true };
        }
        met.add(identifier);
        if (!dependencies) {
            return { name, version };
        }
        const dependencyEntries = Object.entries(dependencies).sort(([xName], [yName]) => xName.localeCompare(yName));
        return {
            name,
            version,
            requires: dependencyEntries.map(([name, versionRange]) => collect(name, versionRange, met)),
        };
    };
    return collect;
};
const findVersionRange = (cwd, packageName) => {
    const { dependencies = {}, devDependencies = {} } = project_1.readHostPackageConfig(cwd);
    return devDependencies[packageName] || dependencies[packageName];
};
exports.default = async (cwd) => {
    const { object: lock } = lockfile_1.parse(fs.readFileSync(path.join(cwd, 'yarn.lock'), 'utf-8'));
    const { name = 'reskript' } = require('../../../../package.json');
    const installedVersionRange = findVersionRange(cwd, name);
    if (!installedVersionRange) {
        console.error(`${name} is not installed in this project`);
        process.exit(1);
    }
    return collectDependenciesFrom(lock)(name, installedVersionRange);
};
//# sourceMappingURL=dependencyTree.js.map