"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = require("lodash");
const eslint_formatter_pretty_1 = __importDefault(require("eslint-formatter-pretty"));
const script_1 = __importDefault(require("./script"));
const style_1 = __importDefault(require("./style"));
const filterUnwantedReports = (report, cmd) => {
    const omitReactUnsafe = cmd.allowUnsafeReactMethod
        ? ({ ruleId, message }) => ruleId !== 'camelcase' || !message.startsWith('Identifier \'UNSAFE')
        : lodash_1.stubTrue;
    const filterMessage = (report) => {
        const messages = report.messages.filter(omitReactUnsafe);
        return { ...report, messages };
    };
    return report.map(filterMessage);
};
exports.default = async (files, cmd) => {
    const [scriptResults, styleResults] = await Promise.all([script_1.default(files, cmd), style_1.default(files, cmd)]);
    const lintResults = filterUnwantedReports([...scriptResults, ...styleResults], cmd);
    const isCleanLint = lodash_1.sumBy(lintResults, ({ errorCount, warningCount }) => errorCount + warningCount) === 0;
    if (isCleanLint) {
        console.log(chalk_1.default.green.bold('(๑ơ ₃ ơ)♥ Great! This is a clean lint over hundreds of rules!'));
    }
    else {
        const output = eslint_formatter_pretty_1.default(lintResults);
        console.log(output);
        process.exit(1);
    }
};
//# sourceMappingURL=run.js.map