"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const pkg_dir_1 = __importDefault(require("pkg-dir"));
const stylelint_1 = require("stylelint");
const lodash_1 = require("lodash");
const config_1 = require("../../../config");
const utils_1 = require("../../utils");
const adaptStyleErrorToScriptError = (message) => {
    const { line, column, rule, severity, text } = message;
    const severityInNumber = severity === 'error' ? 2 : 1;
    return {
        line,
        column,
        ruleId: rule,
        severity: severityInNumber,
        message: text,
        nodeType: '',
        source: null,
    };
};
const adaptStyleResultToScriptResult = (result) => {
    const { source, warnings, parseErrors } = result;
    const errorCount = warnings.length + parseErrors.length;
    return {
        errorCount,
        filePath: source,
        messages: [...parseErrors, ...warnings].map(adaptStyleErrorToScriptError),
        warningCount: 0,
        fixableErrorCount: 0,
        fixableWarningCount: 0,
    };
};
exports.default = async (files, cmd) => {
    const lintingFiles = await utils_1.resolveLintFiles('style', files, cmd);
    if (lodash_1.isEmpty(lintingFiles)) {
        return [];
    }
    const lintConfig = {
        config: config_1.getStyleLintConfig(),
        files: lintingFiles,
        // 当前stylelint的cache有问题，如果一个文件有语法错误，则第一次会被检查出来，第二次错误就消失了
        cache: false,
        cacheLocation: path.join(pkg_dir_1.default.sync(), 'node_modules', '.cache', '.stylelintcache'),
    };
    const report = await stylelint_1.lint(lintConfig);
    return report.results.map(adaptStyleResultToScriptResult);
};
//# sourceMappingURL=style.js.map