"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack_1 = __importDefault(require("webpack"));
const webpack_dev_server_1 = __importDefault(require("webpack-dev-server"));
const build_1 = require("../../../lib/build");
const project_1 = require("../../../lib/project");
const config_1 = __importDefault(require("../dev/config"));
exports.default = async (target, cmd) => {
    process.env.NODE_ENV = 'development';
    const devCmd = {
        ...cmd,
        mode: 'development',
        src: 'src',
    };
    // 为了让用户自定义的插件可以读到足够的信息，这里要把信息造出来
    const projectSettings = project_1.readProjectSettings(devCmd, 'play');
    const { name: hostPackageName } = project_1.readHostPackageConfig(cmd.cwd);
    const entry = {
        ...projectSettings,
        hostPackageName,
        usage: 'play',
        mode: 'development',
        cwd: cmd.cwd,
        srcDirectory: devCmd.src,
        buildTarget: cmd.buildTarget || 'dev',
        features: projectSettings.featureMatrix[cmd.buildTarget],
        isDefaultTarget: true,
        buildVersion: 'HEAD',
        entries: [],
        analyze: false,
        buildTime: (new Date()).toISOString(),
        build: {
            ...projectSettings.build,
            reportLintErrors: false,
        },
        devServer: {
            ...projectSettings.devServer,
            port: 9999,
            openPage: '',
            hot: 'all',
        },
    };
    const config = build_1.createPlayConfig(entry, target);
    const devServerConfig = config_1.default(devCmd, entry.devServer, config.devServer, true);
    webpack_dev_server_1.default.addDevServerEntrypoints(config, devServerConfig);
    const compiler = webpack_1.default(config);
    const server = new webpack_dev_server_1.default(compiler, devServerConfig);
    server.listen(entry.devServer.port);
};
//# sourceMappingURL=run.js.map