"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const rimraf_1 = __importDefault(require("rimraf"));
const resolve_1 = require("resolve");
const rollup_1 = require("rollup");
const rollup_plugin_postcss_1 = __importDefault(require("rollup-plugin-postcss"));
const plugin_node_resolve_1 = __importDefault(require("@rollup/plugin-node-resolve"));
const plugin_commonjs_1 = __importDefault(require("@rollup/plugin-commonjs"));
const plugin_json_1 = __importDefault(require("@rollup/plugin-json"));
const rollup_plugin_auto_external_1 = __importDefault(require("rollup-plugin-auto-external"));
const rollup_plugin_babel_1 = __importDefault(require("rollup-plugin-babel"));
const rollup_plugin_eslint_1 = require("rollup-plugin-eslint");
const rollup_plugin_terser_1 = require("rollup-plugin-terser");
const plugin_replace_1 = __importDefault(require("@rollup/plugin-replace"));
const rollup_plugin_typescript2_1 = __importDefault(require("rollup-plugin-typescript2"));
const lodash_1 = require("lodash");
const config_1 = require("../../../config");
const utils_1 = require("../../../lib/utils");
const project_1 = require("../../../lib/project");
const MODULE_EXTENSIONS = ['.js', '.json', '.ts', '.tsx', '.d.ts'];
const CSS_EXTRACT = {
    index: 'style/index.css',
    extract: true,
    inject: false,
};
// 这些模块不使用`namedExports`是无法通过构建的，所以直接在构建的时候去动态调用一下看上面有多少的导出
const REQUIRE_NAMED_EXPORTS_PACKAGES = ['react', 'react-dom', 'lodash'];
const findEntry = (cwd) => {
    const TARGET_FILE_EXTENSIONS = ['js', 'ts', 'tsx'];
    const fileExtension = TARGET_FILE_EXTENSIONS.find(fileExtension => fs.existsSync(path.join(cwd, 'src', `index.${fileExtension}`)));
    if (fileExtension) {
        return `src/index.${fileExtension}`;
    }
    throw new Error(`Cannot find valid entry. Expecting "src/index.{${TARGET_FILE_EXTENSIONS.join(',')}}".`);
};
exports.default = async ({ cwd, mode, clean }) => {
    process.env.NODE_ENV = mode;
    if (clean) {
        rimraf_1.default.sync(path.join(cwd, 'es'));
        rimraf_1.default.sync(path.join(cwd, 'cjs'));
    }
    const { rollup: { namedDependencyExports = {}, styleOutput = 'index', target = 'universal', }, } = project_1.readProjectSettings({ cwd }, 'rollup');
    const babelConfigOptions = {
        cwd,
        mode,
        usage: 'build',
        browserSupport: {
            chrome: '50',
            safari: '10',
        },
        styleName: false,
        hot: 'none',
        polyfill: false,
        hostType: 'library',
    };
    const eslintOptions = {
        cwd,
        baseConfig: config_1.getScriptLintConfig(),
        rulePaths: [path.resolve(__dirname, '..', '..', '..', 'config', 'lint', 'rules', 'void')],
        throwOnError: true,
        include: 'src/**/*.js',
    };
    const namedExports = REQUIRE_NAMED_EXPORTS_PACKAGES.reduce((namedExports, packageName) => {
        try {
            const moduleEntry = resolve_1.sync(packageName, { basedir: cwd });
            const packageExports = require(moduleEntry);
            return {
                ...namedExports,
                [packageName]: Object.keys(packageExports),
            };
        }
        catch (_a) {
            return {};
        }
    }, namedDependencyExports);
    const cssOptions = {
        extract: CSS_EXTRACT[styleOutput],
        minimize: true,
        use: [
            [
                'less',
                {
                    javascriptEnabled: true,
                    paths: project_1.findUpAllNodeModulesDirectories(cwd),
                },
            ],
        ],
    };
    const plugins = [
        rollup_plugin_eslint_1.eslint(eslintOptions),
        plugin_replace_1.default(utils_1.toDefines(process.env, 'process.env')),
        plugin_json_1.default({ preferConst: true, indent: '    ' }),
        rollup_plugin_postcss_1.default(cssOptions),
        plugin_node_resolve_1.default({ extensions: MODULE_EXTENSIONS, browser: target === 'web' }),
        rollup_plugin_typescript2_1.default(),
        // 在monorepo中依赖可能被提升到根，所以不在当前包的`node_modules`下，因此这里要用正则去找
        plugin_commonjs_1.default({ namedExports, include: /node_modules/ }),
        rollup_plugin_auto_external_1.default({ packagePath: cwd, dependencies: false }),
        rollup_plugin_babel_1.default({ ...config_1.getBabelConfig(babelConfigOptions), exclude: /node_modules/, extensions: MODULE_EXTENSIONS }),
        // 在实际开发中发现`mangle`配置会导致大量的运行时BUG，因此暂时关掉
        mode === 'production' && rollup_plugin_terser_1.terser({ mangle: false }),
    ];
    const inputOptions = {
        context: cwd,
        input: findEntry(cwd),
        plugins: lodash_1.compact(plugins),
        // TODO: 如果不在这里写一个啥，直接用空数组或者没有这个属性，会导致编译失败，原因未知
        external: ['react'],
    };
    const bundle = await rollup_1.rollup(inputOptions);
    bundle.write({ format: 'cjs', file: 'cjs/index.js', sourcemap: true });
    bundle.write({ format: 'es', file: 'es/index.js', sourcemap: true });
};
//# sourceMappingURL=run.js.map