"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const child_process_1 = __importDefault(require("child_process"));
const util_1 = require("util");
const lodash_1 = require("lodash");
const sort_package_json_1 = __importDefault(require("sort-package-json"));
const project_1 = require("../../../lib/project");
const utils_1 = require("./utils");
const exec = util_1.promisify(child_process_1.default.exec);
const writeFile = util_1.promisify(fs.writeFile);
exports.config = async ({ ts, type }) => {
    const tasks = [
        utils_1.putTemplate('.eslintrc.js', process.cwd()),
        ts && utils_1.putTemplate('tsconfig.json', process.cwd()),
        !ts && utils_1.putTemplate('jsconfig.json', process.cwd()),
        !ts && utils_1.putTemplate('webpack.config.js', process.cwd()),
        type === 'app' && utils_1.putTemplate('settings.js', process.cwd()),
    ];
    await Promise.all(lodash_1.compact(tasks));
};
exports.install = async ({ ut }) => {
    const dependencies = [
        ut === 'react' && 'enzyme',
        'webpack',
        'husky',
        'classnames',
    ];
    await exec(`yarn add -D ${lodash_1.compact(dependencies).join(' ')}`);
};
exports.updatePackageInfo = async ({ type }) => {
    const huskyPatch = {
        husky: {
            hooks: {
                'pre-commit': 'npm run lint-staged',
            },
        },
    };
    const libraryPatch = {
        main: 'cjs/index.js',
        module: 'es/index.js',
        sideEffects: false,
        scripts: {
            lint: 'skr lint src demo *.js',
            'lint-staged': 'npm run lint -- --staged',
            build: 'skr rollup --clean',
            start: 'skr dev --src=demo',
            test: 'skr test --coverage',
            prepublishOnly: 'npm run test &&& npm run build',
        },
        files: [
            'cjs',
            'es',
        ],
    };
    const appPatch = {
        scripts: {
            lint: 'skr lint',
            'lint-staged': 'npm run lint -- --staged',
            build: 'skr build --clean',
            start: 'skr dev',
            test: 'skr test --coverage',
        },
    };
    const packageInfo = project_1.readHostPackageConfig(process.cwd());
    const patch = type === 'app' ? appPatch : libraryPatch;
    const newPackageInfo = { ...packageInfo, ...patch, ...huskyPatch };
    await writeFile('./package.json', JSON.stringify(sort_package_json_1.default(newPackageInfo), null, '  ') + '\n', 'utf-8');
};
exports.updateGitIgnore = async ({ type }) => {
    const ignores = (() => {
        if (fs.existsSync('.gitignore')) {
            const content = fs.readFileSync('.gitignore', 'utf-8');
            return content.trim().split('\n');
        }
        return [];
    })();
    const additionalIgnores = type === 'app' ? ['dist'] : ['/es', '/cjs'];
    const newIgnores = additionalIgnores.reduce((ignores, value) => {
        if (ignores.some(line => line.includes(value))) {
            return ignores;
        }
        return [...ignores, value];
    }, ignores);
    await writeFile('.gitignore', newIgnores.join('\n') + '\n', 'utf-8');
};
//# sourceMappingURL=tasks.js.map