"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const lodash_1 = require("lodash");
const jest_cli_1 = require("jest-cli");
const config_1 = require("../../../config");
const project_1 = require("../../../lib/project");
const resolveJestConfig = (jestConfigOptions) => {
    const { cwd } = jestConfigOptions;
    // find out jest.config
    const jestConfigFile = path.resolve(cwd, 'jest.config.js');
    // if no jest config ,return getJestConfig
    if (!fs.existsSync(jestConfigFile)) {
        return JSON.stringify(config_1.getJestConfig(jestConfigOptions));
    }
    const jestConfig = require(jestConfigFile);
    if ('preset' in jestConfig) {
        // if jest config has preset, return jest config
        return JSON.stringify(jestConfig);
    }
    else {
        // if jest config has no preset, return merged
        const skrConfig = config_1.getJestConfig(jestConfigOptions);
        // 用户的覆盖skr的
        return JSON.stringify(lodash_1.merge(skrConfig, jestConfig));
    }
};
exports.default = async (cmd) => {
    const { coverage, watch, cwd, src, target, changedSince } = cmd;
    const argv = [];
    if (coverage) {
        argv.push('--coverage');
    }
    if (watch) {
        argv.push('--watch');
    }
    if (changedSince) {
        argv.push('--changedSince', changedSince);
    }
    const { test: settings, featureMatrix: { dev: features } } = project_1.readProjectSettings(cmd, 'test');
    // featureMatrix 目前以dev为默认目标，以后可以传入--test-target？
    const jestConfigOptions = { cwd, src, target, settings, features };
    const config = resolveJestConfig(jestConfigOptions);
    argv.push('--config', config);
    jest_cli_1.run(argv);
};
//# sourceMappingURL=run.js.map