#! /usr/bin/env node
"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const commander_1 = __importDefault(require("commander"));
const yargs_1 = require("yargs");
const username_1 = require("username");
const os_name_1 = __importDefault(require("os-name"));
const globby_1 = __importDefault(require("globby"));
const semver_1 = __importDefault(require("semver"));
const chalk_1 = __importDefault(require("chalk"));
const unixify_1 = __importDefault(require("unixify"));
const Sentry = __importStar(require("@sentry/node"));
const project_1 = require("../lib/project");
const buildCommand = async (route) => {
    const { command, description, args, run } = await Promise.resolve().then(() => __importStar(require(route)));
    const commandConfig = commander_1.default.command(command);
    commandConfig.description(description);
    args.forEach(option => commandConfig.option(option[0], option[1], option[2]));
    commandConfig.action(run);
};
const readPackageInfo = () => {
    const content = fs.readFileSync(path.join(__dirname, '..', '..', 'package.json'), 'utf-8');
    return JSON.parse(content);
};
const initializeTracking = () => {
    if (process.env.RESKRIPT_TRACK !== 'on') {
        return;
    }
    const { version } = readPackageInfo();
    Sentry.init({ dsn: 'https://000eb745b008478a8028f43f9dda4fbe@sentry.io/1446419', release: version });
    const cwd = yargs_1.argv.cwd || process.cwd();
    const { name } = project_1.readHostPackageConfig(cwd);
    const configure = (scope) => {
        scope.setUser({ username: username_1.sync() });
        scope.setTag('app', name);
        scope.setTag('os', os_name_1.default());
    };
    Sentry.configureScope(configure);
};
const main = async () => {
    if (semver_1.default.lt(process.version, '8.9.0')) {
        console.error(chalk_1.default.yellow('Require node >= v8.9.0 to be installed'));
        process.exit(1);
    }
    process.on('unhandledRejection', e => {
        console.error(e);
        process.exit(2);
    });
    initializeTracking();
    const routes = await globby_1.default(`${unixify_1.default(__dirname)}/commands/*`, { expandDirectories: false, onlyFiles: false });
    await Promise.all(routes.map(buildCommand));
    commander_1.default.parse(process.argv);
};
main();
//# sourceMappingURL=index.js.map