"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const globby_1 = __importDefault(require("globby"));
const g_status_1 = __importDefault(require("g-status"));
const lodash_1 = require("lodash");
const project_1 = require("../../lib/project");
const TYPE_TO_EXTENSIONS = {
    script: ['.js', '.jsx', '.ts', '.tsx'],
    style: ['.css', '.less'],
};
const isStaged = ({ index }) => index !== '?' && index !== ' ';
const findChangedFiles = async (paths) => {
    const files = await g_status_1.default({ path: paths });
    const extractTargetFile = (item) => {
        const { path, index, workingTree } = item;
        const status = index === ' ' ? workingTree : index;
        switch (status) {
            case 'R':
                return {
                    ...item,
                    path: path.split('->')[1].trim(),
                };
            case 'D':
                return [];
            default:
                return item;
        }
    };
    return lodash_1.flatMap(files, extractTargetFile);
};
exports.resolveLintFiles = async (type, files, { staged, changed }) => {
    const extensions = TYPE_TO_EXTENSIONS[type];
    if (staged || changed) {
        // 当前目录可能不在git根目录下，所有的路径要相应做一次修复
        const gitRoot = project_1.findGitRoot() || process.cwd();
        const cwdRelative = path.relative(gitRoot, process.cwd());
        // TODO: 默认仅检查当前目录下的文件，后续考虑提供`--root`参数来强制检查包的根目录下的所有文件
        const paths = extensions.map(extension => `${cwdRelative ? cwdRelative + '/' : ''}*${extension}`);
        const files = await findChangedFiles(paths);
        return files.filter(staged ? isStaged : () => true).map(entry => path.relative(cwdRelative, entry.path));
    }
    const pathToGlob = (file) => {
        // 当前目录比较特殊，只要检查文件，不用去检查子目录
        if (file === '.') {
            return extensions.map(extension => `./*${extension}`);
        }
        if (path.extname(file)) {
            return file;
        }
        return extensions.map(extension => `${file}/**/*${extension}`);
    };
    const fileOrFolders = lodash_1.isEmpty(files) ? ['.', 'src'] : files;
    const globs = lodash_1.flatMap(fileOrFolders, pathToGlob);
    const matchedFiles = await Promise.all(globs.map(pattern => globby_1.default(pattern)));
    return lodash_1.flatten(matchedFiles)
        .filter(filename => !filename.includes('__tests__/'))
        .filter(filename => extensions.includes(path.extname(filename)));
};
//# sourceMappingURL=index.js.map