"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const lodash_1 = require("lodash");
const resolve_1 = require("resolve");
const style_1 = require("./style");
// https://github.com/babel/babel/issues/10379#issuecomment-527077992
const coreJSPreset = () => {
    return {
        plugins: [
            [
                resolve_1.sync('../lib/infrastructure/babel-plugin-resolve-core-js'),
                {
                    replacementName: '$internal/core-js',
                },
            ],
        ],
    };
};
exports.getParseOnlyBabelConfig = (options) => {
    const { browserSupport, usage, polyfill = false, hostType = 'library' } = options;
    // 测试的时候默认自带`core-js`，不需要babel处理
    const requirePolyfill = usage !== 'test' && polyfill && hostType === 'application';
    const presets = [
        requirePolyfill && coreJSPreset,
        [
            resolve_1.sync('@babel/preset-env'),
            {
                debug: false,
                targets: browserSupport,
                modules: usage === 'test' ? 'commonjs' : false,
                useBuiltIns: requirePolyfill ? 'usage' : false,
                corejs: requirePolyfill ? { version: 3, proposals: true } : undefined,
            },
        ],
        resolve_1.sync('@babel/preset-typescript'),
        resolve_1.sync('@babel/preset-react'),
    ];
    const plugins = [
        [resolve_1.sync('@babel/plugin-proposal-decorators'), { legacy: true }],
        resolve_1.sync('@babel/plugin-proposal-class-properties'),
        resolve_1.sync('@babel/plugin-proposal-do-expressions'),
        // export Foo from './Foo';
        resolve_1.sync('@babel/plugin-proposal-export-default-from'),
        // export {Foo} from './Foo';
        resolve_1.sync('@babel/plugin-proposal-export-namespace-from'),
        // const foo = obejct.foo ?? 'default';
        resolve_1.sync('@babel/plugin-proposal-nullish-coalescing-operator'),
        // 1_234_567
        resolve_1.sync('@babel/plugin-proposal-numeric-separator'),
        // object?.foo?.bar
        resolve_1.sync('@babel/plugin-proposal-optional-chaining'),
        // const result = array |> unique |> compact |> flatten
        [resolve_1.sync('@babel/plugin-proposal-pipeline-operator'), { proposal: 'minimal' }],
        // const valid = input.isValid() || throw new Error('Invalid')
        resolve_1.sync('@babel/plugin-proposal-throw-expressions'),
        resolve_1.sync('@babel/plugin-syntax-dynamic-import'),
        resolve_1.sync('@babel/plugin-syntax-import-meta'),
    ];
    return {
        plugins,
        presets: lodash_1.compact(presets),
    };
};
exports.getTransformBabelConfig = (options) => {
    const minimal = exports.getParseOnlyBabelConfig(options);
    const { mode } = options;
    const plugins = [
        // 这东西必须放在最前面，不然`export default class`会被其它插件转义掉没机会确认真实的名字
        resolve_1.sync('../lib/infrastructure/babel-plugin-add-react-display-name'),
        ...minimal.plugins || [],
        [
            resolve_1.sync('babel-plugin-import'),
            {
                libraryName: 'antd',
                libraryDirectory: 'es',
                style: true,
            },
        ],
        mode === 'production' && [resolve_1.sync('babel-plugin-lodash'), { id: ['lodash', 'lodash-decorators'] }],
        resolve_1.sync('babel-plugin-react-require'),
    ];
    return {
        presets: minimal.presets,
        plugins: lodash_1.compact(plugins),
    };
};
exports.getBabelConfig = (options) => {
    const { cwd, srcDirectory = 'src', mode, usage, styleName, hot, hostType, autoRouteSplit = false } = options;
    const transform = exports.getTransformBabelConfig(options);
    const requireHotLoadeer = (usage === 'devServer' || usage === 'play') && hot === 'all';
    const requireReactOptimization = mode === 'production' && hostType === 'application';
    const styleNamePlugin = [
        resolve_1.sync('babel-plugin-react-css-modules'),
        {
            context: cwd,
            exclude: 'node_modules',
            filetypes: {
                '.less': {
                    syntax: 'postcss-less',
                },
            },
            generateScopedName: style_1.generateScopedStyleName,
        },
    ];
    const routeSplitPlugin = [
        resolve_1.sync('../lib/infrastructure/babel-plugin-route-code-split'),
        { cwd, srcDirectory },
    ];
    // 考虑到生成的chunk的hash稳定性，此处不使用`babel-plugin-lodash`来缩减lodash的体积了
    const plugins = [
        ...transform.plugins || [],
        // 由于构建等是通过`reskript`包进行的，因此在这个过程中所有`resolve`调用都会在这个包下开始查找，
        // 而`babel-plugin-react-css-modules`是会调用`resolve`来分析相应的样式表依赖，这会导致找不到文件
        //
        // ```
        // Cannot find module 'reset-css/reset.less'
        // ```
        //
        // 为了让整个`resolve`过程从项目（不是这个包）的`node_modules`下开始找，需要`module-resolver`这个插件，
        // 但是这个插件如果应用于所有的模块，则会影响其它插件的工作（比如`babel-plugin-import`完全失效），
        // 因此配置该插件仅对**非相对路径的样式文件**生效
        [
            resolve_1.sync('babel-plugin-module-resolver'),
            {
                resolvePath(sourcePath) {
                    if (sourcePath.startsWith('.')) {
                        return null;
                    }
                    const extension = path.extname(sourcePath);
                    if ((extension === '.less' || extension === '.css') && sourcePath.startsWith('@/')) {
                        return path.join(cwd, 'src', sourcePath.slice(2));
                    }
                    return null;
                },
            },
        ],
        styleName && styleNamePlugin,
        autoRouteSplit && routeSplitPlugin,
        requireReactOptimization && resolve_1.sync('babel-plugin-transform-react-remove-prop-types'),
        requireHotLoadeer && [resolve_1.sync('react-refresh/babel'), { skipEnvCheck: true }],
    ];
    return { presets: transform.presets, plugins: lodash_1.compact(plugins) };
};
//# sourceMappingURL=babel.js.map