"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const unixify_1 = __importDefault(require("unixify"));
const STATIC_EXTENSIONS = [
    'jpg', 'jpeg', 'png', 'gif', 'svg', 'webp',
    'eot', 'otf', 'ttf', 'woff', 'woff2',
    'mp4', 'webm', 'wav', 'mp3', 'm4a', 'aac', 'oga',
];
const reactJestConfig = (configBasePath, jestConfigOptions) => ({
    moduleNameMapper: {
        '\\.(css|less)$': 'identity-obj-proxy',
        [`\\.(${STATIC_EXTENSIONS.join('|')})$`]: `${configBasePath}/mock`,
        '@/(.*)$': '<rootDir>/src/$1',
    },
    testEnvironment: 'jsdom',
    setupFiles: [`${configBasePath}/setUp`],
    snapshotSerializers: ['enzyme-to-json/serializer'],
    globals: {
        $build: {
            mode: 'production',
        },
        $features: jestConfigOptions && jestConfigOptions.features || {},
    },
    transformIgnorePatterns: [
        'node_modules/(?!@baidu/etui/)[^/]+?/(?!(es|node_modules)/)',
    ],
});
/**
 * configs for node projects
 */
const nodeJestConfig = {
    testEnvironment: 'node',
};
exports.getJestPresetConfig = (target, configBasePath, jestConfigOptions) => {
    const baseConfig = {
        moduleDirectories: ['src', 'node_modules'],
        moduleFileExtensions: ['js', 'ts', 'jsx', 'tsx', 'd.ts'],
        transform: {
            '^.+\\.(js|jsx|ts|tsx)$': `${unixify_1.default(configBasePath)}/transformer`,
        },
        coverageReporters: ['json-summary', 'lcov', 'text', 'clover'],
        testMatch: ['**/__tests__/**/*.test.{js,jsx,ts,tsx}'],
        collectCoverageFrom: ['<rootDir>/src/**/*.{js,jsx,ts,tsx}'],
        coveragePathIgnorePatterns: ['/node_modules/', '/__tests__/'],
    };
    const targetConfig = target === 'react' ? reactJestConfig(configBasePath, jestConfigOptions) : nodeJestConfig;
    return lodash_1.merge(baseConfig, targetConfig);
};
//# sourceMappingURL=preset.js.map