"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const tty_table_1 = __importDefault(require("tty-table"));
const utils_1 = require("./utils");
const drawFeatureMatrix = (buildFeatures) => {
    const buildTargets = Object.keys(buildFeatures);
    const headers = [
        { value: '', align: 'left' },
        ...buildTargets.map(target => ({ value: target })),
    ];
    const featureNames = Object.keys(Object.values(buildFeatures)[0]);
    if (lodash_1.isEmpty(featureNames)) {
        return;
    }
    const nameToValues = (name) => [
        name,
        ...buildTargets.map(target => buildFeatures[target][name]).map(utils_1.printableValue),
    ];
    const rows = featureNames.map(nameToValues);
    const table = new tty_table_1.default(headers, rows);
    console.log(table.render());
};
const createWebpackConfigForEnv = (currentEntry) => {
    const { analyze, build: { thirdParty } } = currentEntry;
    const config = utils_1.createWebpackConfig(currentEntry, lodash_1.compact([thirdParty && 'external', analyze && 'analyze']));
    return config;
};
const buildWithoutFeatureMatrix = (buildEnv) => {
    const currentEntry = {
        ...buildEnv,
        features: {},
        buildTarget: 'default',
        isDefaultTarget: true,
    };
    const config = createWebpackConfigForEnv(currentEntry);
    return [config];
};
const buildWithFeatureMatrix = (buildEnv) => {
    const { buildTarget, featureMatrix, analyze, build: { excludeFeatures }, } = buildEnv;
    const buildFeatures = lodash_1.omit(featureMatrix, excludeFeatures);
    utils_1.checkFeatureMatrixSchema(buildFeatures);
    drawFeatureMatrix(buildFeatures);
    // 由于每一个构建对应的`buildTarget`是不一样的，这会涉及到2个问题：
    //
    // - `settings.js`里的`addition`依赖`buildTarget`
    // - `build.target`的替换同样依赖之，这个可能被用于Sentry等配置上
    //
    // 因此Feature的合并事实上是没有办法做的
    const targetToConfig = ([targetName, features], i) => {
        const currenetEntry = {
            ...buildEnv,
            features,
            buildTarget: targetName,
            // 在`--analyze`的情况下，只让一个feature集合出分析结果
            isDefaultTarget: targetName === buildTarget || (analyze && i === 0),
        };
        return createWebpackConfigForEnv(currenetEntry);
    };
    return Object.entries(buildFeatures).map(targetToConfig);
};
exports.default = (env) => {
    const buildEnv = utils_1.createBuildEnv(env);
    const { cwd, build: { thirdParty, reportLintErrors } } = buildEnv;
    if (!reportLintErrors) {
        utils_1.checkPreCommitHookWhenLintDisabled(cwd);
    }
    return thirdParty ? buildWithoutFeatureMatrix(buildEnv) : buildWithFeatureMatrix(buildEnv);
};
//# sourceMappingURL=build.js.map