"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const resolve_1 = require("resolve");
const monaco_editor_webpack_plugin_1 = __importDefault(require("monaco-editor-webpack-plugin"));
const lodash_1 = require("lodash");
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const loaders_1 = require("../loaders");
const utils_1 = require("./utils");
const guessComponentTypeName = (target) => {
    const filename = path.basename(target, path.extname(target));
    if (/[A-Z]/.test(filename)) {
        return filename;
    }
    return path.basename(path.dirname(target));
};
exports.default = (entry, target) => {
    const buildEntry = utils_1.createDevEntry(entry);
    const baseConfig = utils_1.createWebpackConfig(buildEntry, ['server']);
    const baseRules = baseConfig.module ? baseConfig.module.rules : [];
    const basePlugins = baseConfig.plugins || [];
    const playEntryPath = resolve_1.sync('../../../assets/playground-entry.js.tpl');
    const componentTypeName = guessComponentTypeName(target);
    const entryLoaders = [
        loaders_1.babel(entry),
        {
            loader: resolve_1.sync('../../infrastructure/play-entry-loader'),
            options: {
                componentTypeName,
                componentModulePath: path.resolve(entry.cwd, target),
                extraResources: entry.play.extraResources || [],
            },
        },
    ];
    const htmlConfig = {
        template: resolve_1.sync('../../../assets/playground-entry.ejs'),
        title: `Playground - ${componentTypeName}`,
    };
    return {
        ...baseConfig,
        entry: {
            index: playEntryPath,
        },
        module: {
            rules: [
                {
                    test: playEntryPath,
                    use: lodash_1.compact(entryLoaders),
                },
                ...baseRules,
            ],
        },
        plugins: [
            // 所有用于其它场合的HTML生成都是没用的
            ...basePlugins.filter(p => !(p instanceof html_webpack_plugin_1.default)),
            new html_webpack_plugin_1.default(htmlConfig),
            new monaco_editor_webpack_plugin_1.default({ languages: ['javascript', 'typescript'] }),
        ],
    };
};
//# sourceMappingURL=play.js.map