"use strict";
/**
 * @file 基础webpack配置
 * @author zhanglili
 */
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const resolve_1 = require("resolve");
const lodash_1 = require("lodash");
const pkg_dir_1 = __importDefault(require("pkg-dir"));
const webpack_1 = require("webpack");
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const case_sensitive_paths_webpack_plugin_1 = __importDefault(require("case-sensitive-paths-webpack-plugin"));
const copy_webpack_plugin_1 = __importDefault(require("copy-webpack-plugin"));
const stylelint_webpack_plugin_1 = __importDefault(require("stylelint-webpack-plugin"));
const project_1 = require("../../../lib/project");
const config_1 = require("../../../config");
const utils_1 = require("../../utils");
const utils_2 = require("../utils");
const rules = __importStar(require("../rules"));
// 这东西不在类型定义上
const runtimeValue = webpack_1.DefinePlugin.runtimeValue;
const toDynamicDefines = (context, prefix) => {
    const staticDefines = utils_1.toDefines(context, prefix);
    return lodash_1.mapValues(staticDefines, v => runtimeValue(() => v, true));
};
const factory = entry => {
    const { usage, cwd, srcDirectory, hostPackageName, mode, features, buildTarget, buildVersion, buildTime, entries, build: { thirdParty, reportLintErrors, extractCSS, copies = [], }, } = entry;
    const buildInfo = {
        mode,
        version: buildVersion,
        target: buildTarget,
        time: buildTime,
    };
    const defines = {
        ...toDynamicDefines(process.env, 'process.env'),
        ...toDynamicDefines(features, '$features'),
        ...toDynamicDefines(buildInfo, '$build'),
    };
    const styleLintOptions = {
        config: config_1.getStyleLintConfig(),
        emitErrors: true,
        failOnError: mode === 'production',
        files: `${srcDirectory}/**/*.{css,less}`,
    };
    const htmlPlugins = thirdParty ? [] : utils_2.createHTMLPluginInstances(entry);
    const cssOutput = thirdParty ? 'index.css' : '[name].[contenthash].css';
    const plugins = [
        ...htmlPlugins,
        (usage === 'build' && extractCSS) && new mini_css_extract_plugin_1.default({ filename: cssOutput }),
        new case_sensitive_paths_webpack_plugin_1.default(),
        new webpack_1.ContextReplacementPlugin(/moment[\\/]locale$/, /^\.\/(en|zh-cn)$/),
        new webpack_1.DefinePlugin(defines),
        new copy_webpack_plugin_1.default(copies),
        reportLintErrors && usage === 'build' && new stylelint_webpack_plugin_1.default(styleLintOptions),
    ];
    return {
        mode,
        context: cwd,
        entry: entries.reduce((entry, { name, file }) => ({ ...entry, [name]: file }), {}),
        output: {
            path: path.join(cwd, 'dist', 'assets'),
            filename: '[name].[chunkhash].js',
            publicPath: '/assets/',
        },
        module: {
            rules: Object.values(rules).map(rule => rule(entry)),
        },
        resolve: {
            extensions: ['.js', '.jsx', '.ts', '.tsx', '.d.ts'],
            modules: [
                'node_modules',
                ...project_1.findUpAllNodeModulesDirectories(cwd),
                // 由于webpack需要用到一些包可能是在`reskript/node_modules`下的，
                // 因此需要将自己的`node_modules`也一并加入查找的路径中去
                path.join(pkg_dir_1.default.sync(__dirname), 'node_modules'),
            ],
            mainFields: ['browser', 'module', 'main'],
            alias: {
                '@': path.join(cwd, srcDirectory),
                ...hostPackageName ? { [hostPackageName]: path.join(cwd, 'src') } : {},
                // 要对babel转的里面的`$internal/core-js`固定到自带的`core-js@3`，这里不能强行把所有的`core-js`固定，
                // 第三方代码比如`antd`里面有用于`babel-runtime`进一步引用`core-js@2`，所以全固定到`3.x`会死
                '$internal/core-js': path.dirname(resolve_1.sync('core-js')),
                'regenerator-runtime': path.dirname(resolve_1.sync('regenerator-runtime')),
            },
        },
        plugins: lodash_1.compact(plugins),
    };
};
exports.default = factory;
//# sourceMappingURL=base.js.map