"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const internal_ip_1 = __importDefault(require("internal-ip"));
const lodash_1 = require("lodash");
const webpack_1 = require("webpack");
const friendly_errors_webpack_plugin_1 = __importDefault(require("friendly-errors-webpack-plugin"));
const webpackbar_1 = __importDefault(require("webpackbar"));
const react_refresh_webpack_plugin_1 = __importDefault(require("@pmmmwh/react-refresh-webpack-plugin"));
const project_1 = require("../../project");
const utils_1 = require("../utils");
const devHost = internal_ip_1.default.v4.sync();
const getDevServerMessages = (port, openPage = '') => {
    const prefix = 'Your application is running here';
    const messages = [`${prefix}: http://localhost:${port}/${openPage}`];
    if (devHost) {
        const devHostPrefix = 'or';
        const devHostMsg = ' '.repeat(prefix.length - devHostPrefix.length)
            + devHostPrefix + `: http://${devHost}:${port}/${openPage}`;
        messages.push(devHostMsg);
    }
    return messages;
};
const factory = entry => {
    const { cwd, devServer: { hot, port, openPage } } = entry;
    const webpackBarOptions = {
        name: 'reskript',
    };
    const htmlPlugins = utils_1.createHTMLPluginInstances({ ...entry, isDefaultTarget: true });
    const plugins = [
        ...htmlPlugins,
        new webpackbar_1.default(webpackBarOptions),
        new friendly_errors_webpack_plugin_1.default({
            compilationSuccessInfo: {
                messages: getDevServerMessages(port, openPage),
                notes: [],
            },
        }),
        hot === 'none' ? null : new webpack_1.HotModuleReplacementPlugin(),
        hot === 'all' ? new react_refresh_webpack_plugin_1.default({ overlay: false, forceEnable: true }) : null,
    ];
    return {
        output: {
            path: path.join(cwd, 'dist'),
            // 在使用`HotModuleReplacementPlugin`时是无法使用`chunkhash`的，因此在调试时使用普通的`hash`
            filename: '[name].[hash].js',
        },
        resolve: {
            // `webpack-dev-server`需要在构建时注入自己的客户端，而客户端放在`reskript`下面，
            // 从项目的根目录去找客户端需要的依赖（如`ansi-html`就是一个依赖）是找不到的，
            // 因此把`reskript/node_modules`也加到查找的路径里去
            modules: [...project_1.findUpAllNodeModulesDirectories(cwd)],
        },
        plugins: lodash_1.compact(plugins),
    };
};
exports.default = factory;
//# sourceMappingURL=server.js.map