"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const lodash_1 = require("lodash");
const loaders = __importStar(require("../loaders"));
const createUseWith = (entry) => {
    const createLoader = (name) => {
        const factory = loaders[name];
        return factory(entry);
    };
    return (...names) => lodash_1.compact(lodash_1.compact(names).map(createLoader));
};
const projectSource = (cwd) => {
    const projectDirectory = cwd.endsWith(path.sep) ? cwd : cwd + path.sep;
    return {
        and: [
            { include: projectDirectory },
            { exclude: projectDirectory + 'externals' },
            { exclude: new RegExp(lodash_1.escapeRegExp(`${path.sep}node_modules${path.sep}`)) },
        ],
    };
};
// 在第三方代码与项目代码的处理上，使用的策略是“非`cwd`下的全部算第三方代码”，而不是“包含`node_modules`的算第三方”。
//
// 这一逻辑取决于在使用monorepo时的形式，当前monorepo下我们要求被引用的包是构建后的。
exports.script = (entry) => {
    const { usage, cwd, build: { noCompileScripts, styleName } } = entry;
    const use = createUseWith(entry);
    const mayLint = (...loaders) => {
        const loadersToUse = loaders.concat(usage === 'build' && 'eslint');
        return use(...loadersToUse);
    };
    return {
        test: /\.[jt]sx?$/,
        exclude: noCompileScripts,
        oneOf: [
            {
                resource: projectSource(cwd),
                oneOf: [
                    {
                        test: /\.worker\.[jt]sx?$/,
                        use: mayLint('worker', 'babel'),
                    },
                    {
                        use: mayLint('babel', styleName && usage === 'devServer' && 'styleName'),
                    },
                ],
            },
        ],
    };
};
exports.less = (entry) => {
    const { cwd, usage, build: { noModulesStyles, extractCSS } } = entry;
    const use = createUseWith(entry);
    const finalLoader = (usage === 'build' && extractCSS) ? 'cssExtract' : 'style';
    return {
        test: /\.less$/,
        oneOf: [
            {
                test: [
                    ...noModulesStyles,
                    /\.global\.less$/,
                ],
                use: use(finalLoader, 'css', 'postCSS', 'less', 'styleResources'),
            },
            {
                resource: projectSource(cwd),
                use: use('classNames', finalLoader, 'cssModules', 'postCSSModules', 'less', 'styleResources'),
            },
            {
                use: use(finalLoader, 'css', 'postCSS', 'less', 'styleResources'),
            },
        ],
    };
};
exports.css = (entry) => {
    const { usage, build: { extractCSS } } = entry;
    const use = createUseWith(entry);
    return {
        test: /\.css$/,
        use: use((usage === 'build' && extractCSS) ? 'cssExtract' : 'style', 'css', 'postCSS'),
    };
};
exports.image = (entry) => {
    const use = createUseWith(entry);
    return {
        test: /\.(jpe?g|png|gif|svg)$/i,
        use: use('url', 'img'),
    };
};
exports.file = (entry) => {
    const use = createUseWith(entry);
    return {
        test: /\.(eot|ttf|woff|woff2)(\?.+)?$/,
        use: use('url'),
    };
};
//# sourceMappingURL=index.js.map