"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const terser_1 = __importDefault(require("terser"));
const getHTMLConfig = (filename, entry, env) => {
    const { mode, build: { appTitle, favicon, }, buildVersion, buildTime, buildTarget, } = env;
    const { name, config, template } = entry;
    const buildIdentifier = `${buildVersion}/${buildTarget}@${buildTime}`;
    const baseConfig = {
        favicon,
        filename,
        mode,
        buildVersion,
        buildTime,
        buildTarget,
        buildIdentifier,
        title: appTitle,
        meta: {
            viewport: 'width=device-width, initial-scale=1, maximum-scale=1, user-scalable=no',
            'app-version': buildIdentifier,
        },
        chunks: [name],
        minify: {
            collapseWhitespace: true,
            removeComments: true,
            removeRedundantAttributes: true,
            removeScriptTypeAttributes: true,
            removeStyleLinkTypeAttributes: true,
            useShortDoctype: true,
            minifyCSS: true,
            // `html-webpack-plugin`使用的`html-minifier`仅支持UglifyJS压缩ES5代码，为了使用ES6，自行实现压缩逻辑
            minifyJS(text) {
                return terser_1.default.minify(text).code || '';
            },
        },
        ...config,
    };
    return template ? Object.assign(baseConfig, { template }) : baseConfig;
};
const createHTMLPluginWith = (buildEntry) => {
    const { usage } = buildEntry;
    return (suffix) => (entry) => {
        // 在build的时候，因为静态资源是放在`assets`下，这也是`output`配置项的值，所以HTML要放到外面一层，即`../`下，
        // 不过在调试的时候，启动`webpack-dev-server`没有这个情况，所以又不需要`../`
        const filename = (usage === 'build' ? '../' : '') + `${entry.name}${suffix}.html`;
        const pluginConfig = getHTMLConfig(filename, entry, buildEntry);
        return new html_webpack_plugin_1.default(pluginConfig);
    };
};
exports.createHTMLPluginInstances = (buildEntry) => {
    const { isDefaultTarget, buildTarget, entries } = buildEntry;
    const createInstanceWithSuffix = createHTMLPluginWith(buildEntry);
    const pluginsWithinTarget = entries.map(createInstanceWithSuffix('-' + buildTarget));
    const pluginsOfDefault = isDefaultTarget
        ? entries.map(createInstanceWithSuffix(''))
        : [];
    return [...pluginsWithinTarget, ...pluginsOfDefault];
};
//# sourceMappingURL=index.js.map