"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const plugin = ({ types }) => {
    return {
        visitor: {
            ClassDeclaration(path, state) {
                if (utils_1.isClassDefinitionElligible(path)) {
                    utils_1.insertDisplayNameAfter(path, types, state.file.opts.filename);
                }
            },
            ClassExpression(path, state) {
                if (utils_1.isExpressionContextElligible(path) && utils_1.isClassDefinitionElligible(path)) {
                    utils_1.insertDisplayNameAfter(path, types, state.file.opts.filename);
                }
            },
            FunctionDeclaration(path, state) {
                const filename = state.file.opts.filename;
                if (utils_1.isFunctionDefinitionElligible(path, filename)) {
                    utils_1.insertDisplayNameAfter(path, types, filename);
                }
            },
            FunctionExpression(path, state) {
                const filename = state.file.opts.filename;
                if (utils_1.isExpressionContextElligible(path) && utils_1.isFunctionDefinitionElligible(path, filename)) {
                    utils_1.insertDisplayNameAfter(path, types, filename);
                }
            },
            ArrowFunctionExpression(path, state) {
                const filename = state.file.opts.filename;
                if (utils_1.isExpressionContextElligible(path) && utils_1.isFunctionDefinitionElligible(path, filename)) {
                    utils_1.insertDisplayNameAfter(path, types, filename);
                }
            },
        },
    };
};
exports.default = plugin;
//# sourceMappingURL=index.js.map