"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
const insertStatementAfterImports_1 = __importDefault(require("./insertStatementAfterImports"));
const state_1 = require("./state");
const currentImportDeclarationForReact = (program) => {
    const body = program.get('body');
    const reactImport = body.find(s => s.isImportDeclaration() && s.get('source').get('value') === 'react');
    return reactImport;
};
const createEmptyReactImport = (program) => {
    const declaration = types_1.importDeclaration([], types_1.stringLiteral('react'));
    return insertStatementAfterImports_1.default(program, declaration);
};
const appendNamedSpecifierToReactImport = (declaration, imported) => {
    const specifiers = declaration.get('specifiers');
    const alreadyImported = specifiers.find(s => s.isImportSpecifier() && s.get('imported').node.name === imported);
    if (alreadyImported) {
        return alreadyImported.get('local').node.name;
    }
    const local = declaration.scope.generateUidIdentifier(imported);
    const specifier = types_1.importSpecifier(local, types_1.identifier(imported));
    declaration.pushContainer('specifiers', specifier);
    return local.name;
};
exports.default = (program, state) => {
    if (state_1.isReactDependenciesReady(state)) {
        return;
    }
    const reactImport = currentImportDeclarationForReact(program) || createEmptyReactImport(program);
    if (!reactImport) {
        throw new Error('Failed to find or add react import to source file');
    }
    const lazyName = appendNamedSpecifierToReactImport(reactImport, 'lazy');
    state_1.registerReactDependencies(state, lazyName);
};
//# sourceMappingURL=importReactDependencies.js.map