"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@babel/types");
const isValidTrackRouteUsage_1 = __importDefault(require("./isValidTrackRouteUsage"));
const importReactDependencies_1 = __importDefault(require("./importReactDependencies"));
const removeStaticImport_1 = __importDefault(require("./removeStaticImport"));
const createLazyComponent_1 = __importDefault(require("./createLazyComponent"));
const state_1 = require("./state");
const requiredAttribute = (attribute) => {
    return attribute.isJSXAttribute() && attribute.get('name').node.name === 'component';
};
const resolveComponentTypeName = (attribute) => {
    const value = attribute.get('value');
    if (!value || !value.isJSXExpressionContainer()) {
        return undefined;
    }
    const expression = value.get('expression');
    return expression.isIdentifier() ? expression.node.name : undefined;
};
const isLazyPossible = (program, state, componentTypeName) => {
    if (state_1.isComponentRegistered(state, componentTypeName)) {
        return true;
    }
    const binding = program.scope.bindings[componentTypeName];
    return binding && binding.path.isImportDefaultSpecifier();
};
const processType = (program, state, componentTypeName) => {
    importReactDependencies_1.default(program, state);
    if (!state_1.isComponentRegistered(state, componentTypeName)) {
        const moduleName = removeStaticImport_1.default(program, componentTypeName);
        if (!moduleName) {
            return undefined;
        }
        const lazyComponentTypeName = createLazyComponent_1.default(program, state, moduleName, componentTypeName);
        state_1.registerLazyComponent(state, componentTypeName, lazyComponentTypeName);
    }
    return state_1.findLazyComponentName(state, componentTypeName);
};
const plugin = () => {
    return {
        visitor: {
            JSXElement(path, state) {
                const openingName = path.get('openingElement').get('name');
                if (!openingName.isJSXIdentifier()) {
                    return;
                }
                const componentTypeName = openingName.node.name;
                const program = path.parentPath.find(n => n.type === 'Program');
                if (!isValidTrackRouteUsage_1.default(componentTypeName, program)) {
                    return;
                }
                const attributes = path.get('openingElement').get('attributes');
                const componentAttribute = attributes.find(requiredAttribute);
                if (!componentAttribute) {
                    return;
                }
                const routedComponentTypeName = resolveComponentTypeName(componentAttribute);
                if (!routedComponentTypeName || !isLazyPossible(program, state, routedComponentTypeName)) {
                    return;
                }
                const lazyName = processType(program, state, routedComponentTypeName);
                if (lazyName) {
                    // 前面的一堆判断已经足够确定这里的类型了，简单粗暴地来
                    const value = componentAttribute.get('value');
                    value.set('expression', types_1.identifier(lazyName));
                    path.get('openingElement').pushContainer('attributes', types_1.jsxAttribute(types_1.jsxIdentifier('wrapSuspense')));
                }
            },
        },
    };
};
exports.default = plugin;
//# sourceMappingURL=index.js.map