"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const babel = __importStar(require("@babel/core"));
const traverse_1 = __importDefault(require("@babel/traverse"));
const config_1 = require("../../config");
const babelConfig = config_1.getParseOnlyBabelConfig({ browserSupport: '', usage: 'build', mode: 'development' });
const parse = (source, babelConfig) => {
    const execute = (resolve) => babel.parse(source, babelConfig, (error, ast) => resolve(error ? null : ast));
    return new Promise(execute);
};
const collectDependencies = async (source, filename) => {
    const ast = await parse(source, { ...babelConfig, filename });
    const result = [false, []];
    if (!ast) {
        return result;
    }
    traverse_1.default(ast, {
        JSXIdentifier({ node, parent }) {
            if (parent.type === 'JSXAttribute' && node.name === 'styleName') {
                result[0] = true;
            }
        },
        ImportDeclaration({ node }) {
            const value = node.source.value;
            // 全局的样式不走CSS Module，不用管依赖关系
            if (value.endsWith('.less') && !value.endsWith('.global.less')) {
                result[1].push(value);
            }
        },
    });
    return result;
};
const loader = function styleNameLoader(content, sourceMap) {
    if (this.cacheable) {
        this.cacheable();
    }
    this.async();
    const resolve = util_1.promisify(this.resolve);
    const reportDependencies = async ([hasStyleName, modules]) => {
        if (hasStyleName) {
            const dependencies = await Promise.all(modules.map(m => resolve(this.context, m)));
            dependencies.forEach(this.addDependency);
        }
        this.callback(null, content, sourceMap);
    };
    collectDependencies(content.toString(), this.resourcePath).then(reportDependencies);
};
exports.default = loader;
//# sourceMappingURL=style-name-loader.js.map