"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const stealthy_require_1 = __importDefault(require("stealthy-require"));
const find_up_1 = require("find-up");
const utils_1 = require("../build/entries/utils");
exports.readProjectSettings = (cmd, commandName) => {
    const settingsLocation = path.join(cmd.cwd, 'settings.js');
    if (!fs.existsSync(settingsLocation)) {
        return utils_1.fillProjectSettings({});
    }
    const { plugins = [], ...clientSettings } = stealthy_require_1.default(require.cache, () => require(path.join(cmd.cwd, 'settings.js')));
    const settings = utils_1.fillProjectSettings(clientSettings);
    const pluginOptions = { ...cmd, command: commandName };
    const enhancedSettings = plugins.reduce((baseSettings, apply) => apply(baseSettings, pluginOptions), settings);
    return enhancedSettings;
};
exports.readHostPackageConfig = (cwd) => {
    const packageConfig = JSON.parse(fs.readFileSync(path.join(cwd, 'package.json'), 'utf-8'));
    return packageConfig;
};
exports.findGitRoot = () => {
    const gitDirectory = find_up_1.sync('.git', { type: 'directory' });
    return gitDirectory && path.dirname(gitDirectory);
};
exports.isMonorepo = (cwd) => {
    const root = exports.findGitRoot() || cwd;
    return fs.existsSync(path.join(root, 'packages')) && !fs.existsSync(path.join(root, 'src'));
};
exports.findMonorepoRoot = (cwd) => {
    const dir = find_up_1.sync('packages', { cwd, type: 'directory' });
    if (!dir) {
        throw new Error('Not in a monorepo project');
    }
    return path.dirname(dir);
};
exports.findUpAllNodeModulesDirectories = (cwd, output = []) => {
    const next = find_up_1.sync('node_modules', { cwd, type: 'directory' });
    if (next) {
        // 假设当前是`/foo/bar/node_modules`，找到的就是前面这个路径，
        // 要找上一个`node_modules`的话，必须给的值是`/foo`，所以是往上2层
        return exports.findUpAllNodeModulesDirectories(path.resolve(next, '..', '..'), [...output, next]);
    }
    return output;
};
//# sourceMappingURL=index.js.map